@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013, 2014 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 extern "C"
 {
 #include <libxfce4util/libxfce4util.h>
+#include <libxfce4ui/libxfce4ui.h>
 }
 
 using namespace WhiskerMenu;
@@ -45,6 +46,52 @@ static void whiskermenu_free(XfcePanelPlugin*, Plugin* whiskermenu)
 	whiskermenu = NULL;
 }
 
+// Wait for grab; allows modifier as shortcut
+// Adapted from http://git.xfce.org/xfce/xfce4-panel/tree/common/panel-utils.c#n122
+static bool panel_utils_grab_available()
+{
+	bool grab_succeed = false;
+
+	// Don't try to get the grab for longer then 1/4 second
+	GdkWindow* root = gdk_screen_get_root_window(xfce_gdk_screen_get_active(NULL));
+	GdkGrabStatus grab_pointer = GDK_GRAB_FROZEN;
+	GdkGrabStatus grab_keyboard = GDK_GRAB_FROZEN;
+	for (guint i = 0; i < (G_USEC_PER_SEC / 4 / 100); ++i)
+	{
+		grab_keyboard = gdk_keyboard_grab(root, true, GDK_CURRENT_TIME);
+		if (grab_keyboard == GDK_GRAB_SUCCESS)
+		{
+			const GdkEventMask pointer_mask = static_cast<GdkEventMask>(GDK_BUTTON_PRESS_MASK
+					| GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK
+					| GDK_LEAVE_NOTIFY_MASK | GDK_POINTER_MOTION_MASK);
+			grab_pointer = gdk_pointer_grab(root, true, pointer_mask, NULL, NULL, GDK_CURRENT_TIME);
+			if (grab_pointer == GDK_GRAB_SUCCESS)
+			{
+				grab_succeed = true;
+				break;
+			}
+		}
+		g_usleep(100);
+	}
+
+	// Release the grab so the menu window can take it
+	if (grab_pointer == GDK_GRAB_SUCCESS)
+	{
+		gdk_pointer_ungrab(GDK_CURRENT_TIME);
+	}
+	if (grab_keyboard == GDK_GRAB_SUCCESS)
+	{
+		gdk_keyboard_ungrab(GDK_CURRENT_TIME);
+	}
+
+	if (!grab_succeed)
+	{
+		g_printerr("xfce4-whiskermenu-plugin: Unable to get keyboard and mouse grab. Menu popup failed.\n");
+	}
+
+	return grab_succeed;
+}
+
 //-----------------------------------------------------------------------------
 
 Plugin::Plugin(XfcePanelPlugin* plugin) :
@@ -115,8 +162,10 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 #endif
 	g_signal_connect_slot(plugin, "remote-event", &Plugin::remote_event, this);
 	g_signal_connect_slot<XfcePanelPlugin*>(plugin, "save", &Plugin::save, this);
+	g_signal_connect_slot<XfcePanelPlugin*>(plugin, "about", &Plugin::show_about, this);
 	g_signal_connect_slot(plugin, "size-changed", &Plugin::size_changed, this);
 
+	xfce_panel_plugin_menu_show_about(plugin);
 	xfce_panel_plugin_menu_show_configure(plugin);
 	xfce_panel_plugin_menu_insert_item(plugin, GTK_MENU_ITEM(wm_settings->command[Settings::CommandMenuEditor]->get_menuitem()));
 
@@ -257,7 +306,7 @@ gboolean Plugin::button_clicked(GtkWidget*, GdkEvent* event)
 	}
 	else
 	{
-		popup_menu(false);
+		popup_menu(false, false);
 	}
 
 	return true;
@@ -300,7 +349,7 @@ void Plugin::orientation_changed(XfcePanelPlugin*, GtkOrientation orientation)
 
 gboolean Plugin::remote_event(XfcePanelPlugin*, gchar* name, GValue* value)
 {
-	if (strcmp(name, "popup"))
+	if (strcmp(name, "popup") || !panel_utils_grab_available())
 	{
 		return false;
 	}
@@ -311,7 +360,7 @@ gboolean Plugin::remote_event(XfcePanelPlugin*, gchar* name, GValue* value)
 	}
 	else
 	{
-		popup_menu(value && G_VALUE_HOLDS_BOOLEAN(value) && g_value_get_boolean(value));
+		popup_menu(value && G_VALUE_HOLDS_BOOLEAN(value) && g_value_get_boolean(value), true);
 	}
 
 	return true;
@@ -331,6 +380,28 @@ void Plugin::save()
 
 //-----------------------------------------------------------------------------
 
+void Plugin::show_about()
+{
+	const gchar* authors[] = {
+		"Graeme Gott <graeme@gottcode.org>",
+		NULL };
+
+	gtk_show_about_dialog
+		(NULL,
+		"authors", authors,
+		"comments", _("Alternate application launcher for Xfce"),
+		"copyright", _("Copyright \302\251 2013-2014 Graeme Gott"),
+		"license", XFCE_LICENSE_GPL,
+		"logo-icon-name", "xfce4-whiskermenu",
+		"program-name", PACKAGE_NAME,
+		"translator-credits", _("translator-credits"),
+		"version", PACKAGE_VERSION,
+		"website", "http://gottcode.org/xfce4-whiskermenu-plugin/",
+		NULL);
+}
+
+//-----------------------------------------------------------------------------
+
 gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 {
 	GtkOrientation panel_orientation = xfce_panel_plugin_get_orientation(m_plugin);
@@ -342,13 +413,19 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 	gint row_size = size;
 #endif
 
+	// Make icon expand to fill button if title is not visible
+	gtk_box_set_child_packing(GTK_BOX(m_button_box), GTK_WIDGET(m_button_icon),
+			!wm_settings->button_title_visible,
+			!wm_settings->button_title_visible,
+			0, GTK_PACK_START);
+
 	// Resize icon
 	GtkStyle* style = gtk_widget_get_style(m_button);
 	gint border = (2 * std::max(style->xthickness, style->ythickness)) + 2;
 	xfce_panel_image_set_size(m_button_icon, row_size - border);
 
 #if (LIBXFCE4PANEL_CHECK_VERSION(4,9,0))
-	if (wm_settings->button_title_visible && (mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR))
+	if (wm_settings->button_title_visible || !wm_settings->button_single_row)
 	{
 		xfce_panel_plugin_set_small(m_plugin, false);
 
@@ -356,9 +433,9 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 		GtkRequisition label_size;
 		gtk_widget_size_request(GTK_WIDGET(m_button_label), &label_size);
 		if (mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR &&
-		    wm_settings->button_title_visible &&
-		    wm_settings->button_icon_visible &&
-		    label_size.width <= (size - row_size))
+				wm_settings->button_title_visible &&
+				wm_settings->button_icon_visible &&
+				label_size.width <= (size - row_size))
 		{
 			orientation = GTK_ORIENTATION_HORIZONTAL;
 		}
@@ -370,32 +447,31 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 #endif
 
 	// Fix alignment in deskbar mode
-        if (panel_orientation == GTK_ORIENTATION_VERTICAL &&
-	    orientation == GTK_ORIENTATION_HORIZONTAL)
+	if ((panel_orientation == GTK_ORIENTATION_VERTICAL) && (orientation == GTK_ORIENTATION_HORIZONTAL))
 	{
-		gtk_box_set_child_packing(m_button_box, GTK_WIDGET(m_button_icon), false, false, 0, GTK_PACK_START);
 		gtk_box_set_child_packing(m_button_box, GTK_WIDGET(m_button_label), false, false, 0, GTK_PACK_START);
 	}
 	else
 	{
-		gtk_box_set_child_packing(m_button_box, GTK_WIDGET(m_button_icon), true, false, 0, GTK_PACK_START);
 		gtk_box_set_child_packing(m_button_box, GTK_WIDGET(m_button_label), true, true, 0, GTK_PACK_START);
 	}
 
 	gtk_orientable_set_orientation(GTK_ORIENTABLE(m_button_box), orientation);
 
-
 	return true;
 }
 
 //-----------------------------------------------------------------------------
 
-void Plugin::popup_menu(bool at_cursor)
+void Plugin::popup_menu(bool at_cursor, bool activate_button)
 {
 	if (!at_cursor)
 	{
 		xfce_panel_plugin_block_autohide(m_plugin, true);
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_button), true);
+		if (activate_button)
+		{
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_button), true);
+		}
 		m_window->show(m_button, xfce_panel_plugin_get_orientation(m_plugin) == GTK_ORIENTATION_HORIZONTAL);
 	}
 	else
